#
#       Script for the standard 2AFC task
#
#           Yasuharu Okamoto, 2019
#
import numpy as np
import pystan
import matplotlib.pyplot as plt
import csv

flnm = input('Input data file (*.csv) = ')
with open(flnm, 'r') as f:
    data = [v for v in csv.reader(f)]

fout_nm = input('Output text file (*.txt) = ')
fout = open(fout_nm, 'w')
fout.write('Input Data File = {}\n'.format(flnm))
    
Freq_NS = []
Freq_NS.append(int(data[1][1]))
Freq_NS.append(int(data[1][2]))
Freq_SN = []
Freq_SN.append(int(data[2][1]))
Freq_SN.append(int(data[2][2]))
print(Freq_NS)
print(Freq_SN)
fout.write('<N, S>: {}'.format(Freq_NS))
fout.write('\n<S, N>: {}'.format(Freq_SN))
TN_NS = sum(Freq_NS)
Rating_NS = np.array(Freq_NS)

TN_SN = sum(Freq_SN)
Rating_SN = np.flip(np.array(Freq_SN))

Data = {'n_NS': Rating_NS, 'n_SN': Rating_SN}

sm = pystan.StanModel(file = 'SDT2AFC2Cat.stan')

fit = sm.sampling(data = Data, n_jobs = 1)

print(fit)
fout.write('\n{}\n'.format(fit))

mu = fit['mu_s']
tau = fit['tau']
p_NS = fit['p_NS']
p_SN = fit['p_SN']

mu_L05 = np.percentile(mu, 5)
mu_med = np.percentile(mu, 50)
mu_U95 = np.percentile(mu, 95)
fout.write("\nd': \nMed. = {0:<.3f},  90%CI = [{1:<.3f}, {2:<.3f}".
           format(mu_med, mu_L05, mu_U95))

tau_L05 = np.percentile(tau, 5)
tau_med = np.percentile(tau, 50)
tau_U95 = np.percentile(tau, 95)
fout.write('\ntau: \nMed. = {0:<.3f},  90%CI ~ [{1:<.3f}, {2:<.3f}]'.
           format(tau_med, tau_L05, tau_U95))

p1 = np.median(p_NS, axis = 0)
p2 = np.median(p_SN, axis = 0)
p2 = [p2[1], p2[0]]

plt.hist(mu)
plt.xlabel("d'($\mu_S$)", fontsize = 14)
plt.title("Posterior Distribution of d'($\mu_S$)" + \
          '\nMed. = {0:<.3f},  90%CI = [{1:M<.3f}, {2:<.3f}]'.
          format(mu_med, mu_L05, mu_U95), fontsize = 16)
plt.show()

plt.hist(tau)
plt.xlabel(r'$\tau$', fontsize = 18)
plt.title(r'Posterior Distribution of $\tau$' + \
          '\nMed. = {0:<.3f},  90%CI = [{1:<.3f}, {2:<.3f}]'.
          format(tau_med, tau_L05, tau_U95), fontsize = 16)
plt.show()

xcat = [1, 2]
xlabels = ['Cat-1', 'Cat-2']
y1 = np.array(Freq_NS) / TN_NS
y2 = np.array(Freq_SN) / TN_SN

plt.plot(xcat, y1, 'b:', linewidth = 10, label = 'Data/<N, S>')
plt.plot(xcat, p1, 'g-', linewidth = 3, label = 'Model/<N, S>')
plt.plot(xcat, y2, 'r:', linewidth = 10, label = 'Data/<S, N>')
plt.plot(xcat, p2, 'm-', linewidth = 3,label = 'Model/<S, N>')
plt.xticks(xcat, xlabels, fontsize = 14)
plt.xlim(0.8, 2.2)
plt.xlabel('Rating Category', fontsize = 14)
plt.ylabel('Probability/Proportion', fontsize = 14)
plt.title('Rating in Conditions <N, S> and <S, N>', fontsize = 18)
plt.legend(loc = 'upper center', fontsize = 10)
plt.show()
      
fout.close()
print('\n' + fout_nm + ' was saved.\n')
