#
#       Number of rating categories should be even.
#
#           Yasuharu Okamoto, 2019.10
#
import numpy as np
import pystan
import matplotlib.pyplot as plt
import csv
import pickle

flnm = input('Input data file (*.csv) = ')
with open(flnm, 'r') as f:
    data = [v for v in csv.reader(f)]

fout_nm = input('Output text file (*.txt) = ')
fout = open(fout_nm, 'w')
fout.write('Input Data File = {}\n'.format(flnm))

K = len(data[0]) - 1
print('K = ', K)
if (K % 2) == 1:
    print('The number of rating categories is not even!')
    import sys
    sys.exit()
    
Freq_NS = []
for k in range(K):
    Freq_NS.append(int(data[1][1 + k]))
Freq_SN = []
for k in range(K):
    Freq_SN.append(int(data[2][1 + k]))
print(Freq_NS)
print(Freq_SN)
fout.write('\n<N, S>\n{}\n'.format(Freq_NS))
fout.write('\n<S, N>\n{}\n'.format(Freq_SN))   
TN_NS = sum(Freq_NS)
Rating_NS = Freq_NS
print(Rating_NS)
TN_SN = sum(Freq_SN)
Rating_SN = np.flip(np.array(Freq_SN))

Data = {'K': K, 'n_NS': Rating_NS, 'n_SN': Rating_SN}

sm = pystan.StanModel(file = 'SDT2AFCEvenCat.stan')

fit = sm.sampling(data = Data, n_jobs = 1)

print(fit)
fout.write('\n{}\n'.format(fit))

mu = fit['mu_s']
tau = fit['tau']
C = fit['C']
p_NS = fit['p_NS']
p_SN = fit['p_SN']

mu_L05 = np.percentile(mu, 5)
mu_med = np.percentile(mu, 50)
mu_U95 = np.percentile(mu, 95)
fout.write("\nd':\nMedian = {0:<.3f},   90%CI = [{1:<.3f}, {2:<.3f}]\n".
           format(mu_med, mu_L05, mu_U95))

tau_L05 = np.percentile(tau, 5)
tau_med = np.percentile(tau, 50)
tau_U95 = np.percentile(tau, 95)
fout.write('\ntau:\nMedian = {0:<.3f},   90%CI = [{1:<.3f}, {2:<.3f}]\n'.
           format(tau_med, tau_L05, tau_U95))

C_L05 = np.zeros(K-1)
C_med = np.zeros(K-1)
C_U95 = np.zeros(K-1)
for k in range(K-1):
    C_L05[k] = np.percentile(C.T[k], 5)
    C_med[k] = np.percentile(C.T[k], 50)
    C_U95[k] = np.percentile(C.T[k], 95)
for k in range(K-1):
    fout.write(('\nC[{0}]:\n' + \
               'Median = {1:<.3f},   90%CI = [{2:<.3f}, {3:<.3f}]\n').
               format(k+1, C_med[k], C_L05[k], C_U95[k]))

plt.hist(mu)
plt.xlabel("d'($\mu_S$)", fontsize = 14)
plt.title("Posterior Distribution of d'($\mu_S$)" + \
          '\nMed. = {0:<.3f},   90%CI = [{1:<.3f}, {2:<.3f}]'.
          format(mu_med, mu_L05, mu_U95), fontsize = 16)
plt.show()

plt.hist(tau)
plt.xlabel(r'$\tau$', fontsize = 18)
plt.title(r'Posterior Distribution of $\tau$' + \
          '\nMed. = {0:<.3f},   90%CI = [{1:<.3f}, {2:<.3f}]'.
          format(tau_med, tau_L05, tau_U95), fontsize = 16)
plt.show()

for k in range(K-1):
    if k + 1 != K/2:
        plt.hist(C.T[k], label = 'C{}'.format(k+1))
plt.title('Posterior Distributions of Category Boundaries' + \
          '\nC{} = 0 fixed'.format(K//2), fontsize = 16)
plt.legend(loc = 'lower center')
plt.show()

xcat = np.arange(1, K+0.1, 1)
xlabels = ['{}'.format(int(v)) for v in xcat]
y1 = np.array(Freq_NS) / TN_NS
y2 = np.array(Freq_SN) / TN_SN

p1 = np.median(p_NS, axis = 0)
p2 = np.median(p_SN, axis = 0)
p2 = np.flip(p2)

plt.plot(xcat, y1, 'b--', linewidth = 3, alpha = 0.7, label = 'Data/<N, S>')
plt.plot(xcat, p1, 'g-', linewidth = 3, alpha = 0.7, label = 'Model/<N, S>')
plt.plot(xcat, y2, 'r--', linewidth = 3, alpha = 0.7, label = 'Data/<S, N>')
plt.plot(xcat, p2, 'm-', linewidth = 3, alpha = 0.7, label = 'Model/<S, N>')

plt.xticks(xcat, xlabels, fontsize = 14)
plt.xlabel('Rating Category', fontsize = 14)
plt.ylabel('Probability/Proportion', fontsize = 14)
plt.title('Ratings in Conditions <N, S> and <S, N>', fontsize = 18)
plt.legend(loc = 'upper center', fontsize = 10)
plt.show()

fout.close()
print('\n{} was saved.\n'.format(fout_nm))
